@@ -173,3 +173,58 @@ pub fn load_overrides(config: Config, opt: &Opt) -> Config {
 
     new_config
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use structopt::StructOpt;
+    use stylua_lib::{CallParenType, IndentType, LineEndings, QuoteStyle};
+
+    #[test]
+    fn test_override_column_width() {
+        let override_opt = Opt::from_iter(vec!["BINARY_NAME", "--column-width", "80"]);
+        let default_config = Config::new();
+        let config = load_overrides(default_config, &override_opt);
+        assert_eq!(config.column_width(), 80);
+    }
+
+    #[test]
+    fn test_override_line_endings() {
+        let override_opt = Opt::from_iter(vec!["BINARY_NAME", "--line-endings", "Windows"]);
+        let default_config = Config::new();
+        let config = load_overrides(default_config, &override_opt);
+        assert_eq!(config.line_endings(), LineEndings::Windows);
+    }
+
+    #[test]
+    fn test_override_indent_type() {
+        let override_opt = Opt::from_iter(vec!["BINARY_NAME", "--indent-type", "Spaces"]);
+        let default_config = Config::new();
+        let config = load_overrides(default_config, &override_opt);
+        assert_eq!(config.indent_type(), IndentType::Spaces);
+    }
+
+    #[test]
+    fn test_override_indent_width() {
+        let override_opt = Opt::from_iter(vec!["BINARY_NAME", "--indent-width", "2"]);
+        let default_config = Config::new();
+        let config = load_overrides(default_config, &override_opt);
+        assert_eq!(config.indent_width(), 2);
+    }
+
+    #[test]
+    fn test_override_quote_style() {
+        let override_opt = Opt::from_iter(vec!["BINARY_NAME", "--quote-style", "ForceSingle"]);
+        let default_config = Config::new();
+        let config = load_overrides(default_config, &override_opt);
+        assert_eq!(config.quote_style(), QuoteStyle::ForceSingle);
+    }
+
+    #[test]
+    fn test_override_call_parentheses() {
+        let override_opt = Opt::from_iter(vec!["BINARY_NAME", "--call-parentheses", "None"]);
+        let default_config = Config::new();
+        let config = load_overrides(default_config, &override_opt);
+        assert_eq!(config.call_parentheses(), CallParenType::None);
+    }
+}
