@@ -1,5 +1,5 @@
 use crate::{
-    context::{create_indent_trivia, create_newline_trivia, Context},
+    context::{create_indent_trivia, create_newline_trivia, Context, FormatNode},
     fmt_symbol,
     formatters::{
         assignment::hang_punctuated_list,
@@ -15,7 +15,7 @@ use crate::{
     shape::Shape,
 };
 use full_moon::ast::{
-    punctuated::Punctuated, Block, Expression, LastStmt, Prefix, Return, Stmt, Var,
+    punctuated::Punctuated, Block, Expression, LastStmt, Prefix, Return, Stmt, Value, Var,
 };
 use full_moon::tokenizer::TokenType;
 use full_moon::tokenizer::{Token, TokenReference};
@@ -28,6 +28,15 @@ macro_rules! update_first_token {
     }};
 }
 
+fn is_function_or_table_constructor(expression: &Expression) -> bool {
+    match expression {
+        Expression::Value { value, .. } => {
+            matches!(&**value, Value::TableConstructor(_) | Value::Function(_))
+        }
+        _ => false,
+    }
+}
+
 pub fn format_return(ctx: &Context, return_node: &Return, shape: Shape) -> Return {
     // Calculate trivia
     let leading_trivia = vec![create_indent_trivia(ctx, shape)];
@@ -60,14 +69,24 @@ pub fn format_return(ctx: &Context, return_node: &Return, shape: Shape) -> Retur
         let (should_format_multiline, singleline_returns) = if contains_comments {
             (true, Punctuated::new())
         } else {
-            // Firstly attempt to format the returns onto a single line, using an infinite column width shape
-            let singleline_returns =
-                format_punctuated(ctx, returns, shape.with_infinite_width(), format_expression);
+            // Special case:
+            // The singleline returns is full of multiline tables or anonymous functions
+            // If so, we should just format inline, normally.
+            if returns.iter().all(is_function_or_table_constructor) {
+                (
+                    false,
+                    format_punctuated(ctx, returns, shape, format_expression),
+                )
+            } else {
+                // Firstly attempt to format the returns onto a single line, using an infinite column width shape
+                let singleline_returns =
+                    format_punctuated(ctx, returns, shape.with_infinite_width(), format_expression);
 
-            // Test the return to see if its over width
-            let singleline_shape =
-                shape + strip_trailing_trivia(&singleline_returns).to_string().len();
-            (singleline_shape.over_budget(), singleline_returns)
+                // Test the return to see if its over width
+                let singleline_shape =
+                    shape + strip_trailing_trivia(&singleline_returns).to_string().len();
+                (singleline_shape.over_budget(), singleline_returns)
+            }
         };
 
         // TODO: this is similar to assignment tactics - can we abstract them into a common function?
@@ -160,7 +179,10 @@ fn format_last_stmt_block(ctx: &Context, last_stmt: &LastStmt, shape: Shape) ->
 }
 
 pub fn format_last_stmt(ctx: &Context, last_stmt: &LastStmt, shape: Shape) -> LastStmt {
-    if !ctx.should_format_node(last_stmt) {
+    let should_format = ctx.should_format_node(last_stmt);
+    if let FormatNode::Skip = should_format {
+        return last_stmt.to_owned();
+    } else if let FormatNode::NotInRange = should_format {
         return format_last_stmt_block(ctx, last_stmt, shape);
     }
 
@@ -399,7 +421,7 @@ pub fn format_block(ctx: &Context, block: &Block, shape: Shape) -> Block {
 
         // If this is the first stmt, then remove any leading newlines
         if !found_first_stmt {
-            if ctx.should_format_node(&stmt) {
+            if let FormatNode::Normal = ctx.should_format_node(&stmt) {
                 stmt = stmt_remove_leading_newlines(stmt);
             }
             found_first_stmt = true;
@@ -484,7 +506,8 @@ pub fn format_block(ctx: &Context, block: &Block, shape: Shape) -> Block {
             let shape = shape.reset();
             let mut last_stmt = format_last_stmt(&ctx, last_stmt, shape);
             // If this is the first stmt, then remove any leading newlines
-            if !found_first_stmt && ctx.should_format_node(&last_stmt) {
+            if !found_first_stmt && matches!(ctx.should_format_node(&last_stmt), FormatNode::Normal)
+            {
                 last_stmt = last_stmt_remove_leading_newlines(last_stmt);
             }
             // LastStmt will never need a semicolon
