@@ -21,6 +21,10 @@ pub fn trivia_is_whitespace(trivia: &Token) -> bool {
     matches!(trivia.token_kind(), TokenKind::Whitespace)
 }
 
+pub fn trivia_is_singleline_comment(trivia: &Token) -> bool {
+    matches!(trivia.token_kind(), TokenKind::SingleLineComment)
+}
+
 pub fn trivia_is_comment(trivia: &Token) -> bool {
     matches!(
         trivia.token_kind(),
@@ -230,8 +234,7 @@ fn var_trailing_trivia(var: &Var) -> Vec<Token> {
             if let Some(last_suffix) = var_expr.suffixes().last() {
                 suffix_trailing_trivia(last_suffix)
             } else {
-                // TODO: is it possible for this to happen?
-                vec![]
+                unreachable!("got a VarExpression with no suffix");
             }
         }
         other => panic!("unknown node {:?}", other),
@@ -249,8 +252,7 @@ pub fn get_value_trailing_trivia(value: &Value) -> Vec<Token> {
             if let Some(last_suffix) = function_call.suffixes().last() {
                 suffix_trailing_trivia(last_suffix)
             } else {
-                // TODO: is it possible for this to happen?
-                vec![]
+                unreachable!("got a FunctionCall with no suffix");
             }
         }
         Value::String(token_reference) => token_reference
@@ -763,11 +765,8 @@ pub fn type_info_leading_trivia(type_info: &TypeInfo) -> Vec<&Token> {
 }
 
 #[cfg(feature = "luau")]
-pub fn take_type_argument_trailing_comments(
-    type_argument: &TypeArgument,
-) -> (TypeArgument, Vec<Token>) {
-    let (type_info, trailing_trivia) =
-        get_type_info_trailing_trivia(type_argument.type_info().to_owned());
+pub fn take_type_info_trailing_comments(type_info: &TypeInfo) -> (TypeInfo, Vec<Token>) {
+    let (type_info, trailing_trivia) = get_type_info_trailing_trivia(type_info.to_owned());
 
     let trailing_comments = trailing_trivia
         .iter()
@@ -778,6 +777,16 @@ pub fn take_type_argument_trailing_comments(
         })
         .collect();
 
+    (type_info, trailing_comments)
+}
+
+#[cfg(feature = "luau")]
+pub fn take_type_argument_trailing_comments(
+    type_argument: &TypeArgument,
+) -> (TypeArgument, Vec<Token>) {
+    let (type_info, trailing_comments) =
+        take_type_info_trailing_comments(type_argument.type_info());
+
     (
         type_argument.to_owned().with_type_info(type_info),
         trailing_comments,
@@ -853,7 +862,7 @@ pub fn get_stmt_trailing_trivia(stmt: Stmt) -> (Stmt, Vec<Token>) {
             let last_suffix = function_call.suffixes().last();
             let trailing_trivia = match last_suffix {
                 Some(suffix) => suffix_trailing_trivia(suffix),
-                None => Vec::new(),
+                None => unreachable!("got a FunctionCall with no suffix"),
             };
 
             (
@@ -1020,13 +1029,29 @@ pub fn get_last_stmt_trailing_trivia(last_stmt: LastStmt) -> (LastStmt, Vec<Toke
     }
 }
 
+#[derive(Clone, Copy)]
+pub enum CommentSearch {
+    // Only care about singleline comments
+    #[allow(dead_code)]
+    Single,
+    // Looking for all comments
+    All,
+}
+
+pub fn trivia_contains_comments<'a>(
+    mut trivia: impl Iterator<Item = &'a Token>,
+    search: CommentSearch,
+) -> bool {
+    let tester = match search {
+        CommentSearch::Single => trivia_is_singleline_comment,
+        CommentSearch::All => trivia_is_comment,
+    };
+
+    trivia.any(tester)
+}
+
 pub fn token_trivia_contains_comments<'a>(trivia: impl Iterator<Item = &'a Token>) -> bool {
-    for trivia in trivia {
-        if trivia_is_comment(trivia) {
-            return true;
-        }
-    }
-    false
+    trivia_contains_comments(trivia, CommentSearch::All)
 }
 
 pub fn token_contains_leading_comments(token_ref: &TokenReference) -> bool {
@@ -1037,15 +1062,26 @@ pub fn token_contains_trailing_comments(token_ref: &TokenReference) -> bool {
     token_trivia_contains_comments(token_ref.trailing_trivia())
 }
 
-pub fn token_contains_comments(token_ref: &TokenReference) -> bool {
-    token_trivia_contains_comments(token_ref.leading_trivia())
-        || token_trivia_contains_comments(token_ref.trailing_trivia())
+pub fn token_contains_comments_search(token: &TokenReference, search: CommentSearch) -> bool {
+    trivia_contains_comments(token.leading_trivia(), search)
+        || trivia_contains_comments(token.trailing_trivia(), search)
+}
+
+pub fn token_contains_comments(token: &TokenReference) -> bool {
+    token_contains_comments_search(token, CommentSearch::All)
 }
 
 pub fn contains_comments(node: impl Node) -> bool {
     node.tokens().into_iter().any(token_contains_comments)
 }
 
+#[allow(dead_code)]
+pub fn contains_singleline_comments(node: impl Node) -> bool {
+    node.tokens()
+        .into_iter()
+        .any(|token| token_contains_comments_search(token, CommentSearch::Single))
+}
+
 /// Checks whether any [`Field`] within a [`TableConstructor`] contains comments, without checking the braces
 pub fn table_fields_contains_comments(table_constructor: &TableConstructor) -> bool {
     table_constructor.fields().pairs().any(|field| {
@@ -1076,28 +1112,7 @@ pub fn table_fields_contains_comments(table_constructor: &TableConstructor) -> b
 // This can only happen if the expression is a BinOp
 // We should ignore any comments which are trailing for the whole expression, as they are not inline
 pub fn expression_contains_inline_comments(expression: &Expression) -> bool {
-    match expression {
-        Expression::BinaryOperator { lhs, binop, rhs } => {
-            contains_comments(binop) || contains_comments(lhs)
-            // Check if the binop chain still continues
-            // If so, we should keep checking the expresion
-            // Otherwise, stop checking
-            || match &**rhs {
-                Expression::BinaryOperator { .. } => expression_contains_inline_comments(rhs),
-                Expression::UnaryOperator { unop, expression } => {
-                    let op_contains_comments = match unop {
-                        UnOp::Minus(token) | UnOp::Not(token) | UnOp::Hash(token) => contains_comments(token),
-                        other => panic!("unknown node {:?}", other)
-                    };
-                    op_contains_comments || expression_contains_inline_comments(expression)
-                }
-                Expression::Value{ .. } => false,
-                Expression::Parentheses { .. } => contains_comments(rhs),
-                other => panic!("unknown node {:?}", other),
-            }
-        }
-        _ => false,
-    }
+    contains_comments(expression.update_trailing_trivia(FormatTriviaType::Replace(vec![])))
 }
 
 // Commonly, we update trivia to add in a newline and indent trivia to the leading trivia of a token/node.
@@ -1131,3 +1146,49 @@ where
 
     node.update_leading_trivia(FormatTriviaType::Replace(leading_trivia))
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn test_token_contains_singleline_comments() {
+        let token = TokenReference::new(
+            vec![],
+            Token::new(TokenType::Symbol {
+                symbol: full_moon::tokenizer::Symbol::And,
+            }),
+            vec![Token::new(TokenType::SingleLineComment {
+                comment: "hello".into(),
+            })],
+        );
+        assert!(contains_singleline_comments(token))
+    }
+
+    #[test]
+    fn test_token_contains_no_singleline_comments() {
+        let token = TokenReference::new(
+            vec![],
+            Token::new(TokenType::Symbol {
+                symbol: full_moon::tokenizer::Symbol::And,
+            }),
+            vec![],
+        );
+        assert!(!contains_singleline_comments(token))
+    }
+
+    #[test]
+    fn test_token_contains_no_singleline_comments_2() {
+        let token = TokenReference::new(
+            vec![],
+            Token::new(TokenType::Symbol {
+                symbol: full_moon::tokenizer::Symbol::And,
+            }),
+            vec![Token::new(TokenType::MultiLineComment {
+                comment: "hello".into(),
+                blocks: 1,
+            })],
+        );
+        assert!(!contains_singleline_comments(token))
+    }
+}
